<?php

namespace common\logic\Feature\Feature\Service;

use common\logic\AbstractManager;
use common\logic\Feature\Feature\Model\Feature;
use common\logic\Feature\Feature\Repository\FeatureRepository;
use common\logic\Feature\FeatureProducer\Repository\FeatureProducerRepository;
use common\logic\Feature\FeatureProducer\Service\FeatureProducerBuilder;
use yii\base\ErrorException;

class FeatureManager extends AbstractManager
{
    protected FeatureRepository $featureRepository;
    protected FeatureProducerRepository $featureProducerRepository;
    protected FeatureBuilder $featureBuilder;
    protected FeatureProducerBuilder $featureProducerBuilder;

    public function loadDependencies(): void
    {
        $this->featureRepository = $this->loadService(FeatureRepository::class);
        $this->featureProducerRepository = $this->loadService(FeatureProducerRepository::class);
        $this->featureBuilder = $this->loadService(FeatureBuilder::class);
        $this->featureProducerBuilder = $this->loadService(FeatureProducerBuilder::class);
    }

    public function isEnabled(string $aliasFeature): bool
    {
        $feature = $this->featureRepository->findOneFeatureByAlias($aliasFeature);
        if(!$feature) {
            throw new ErrorException("Fonctionnalité avec l'alias '".$aliasFeature."' non trouvée");
        }

        if(!$feature->status) {
            return false;
        }

        $featureProducer = $this->featureProducerRepository->findOneFeatureProducer($feature);
        if(!$featureProducer || is_null($featureProducer->status)) {
            if($feature->is_paid_feature || $feature->only_for_selected_producers) {
                return false;
            }

            return $feature->status;
        }
        else {
            return $featureProducer->status;
        }
    }

    public function isDisabled(string $aliasFeature): bool
    {
        return !$this->isEnabled($aliasFeature);
    }

    public function enable(Feature $feature): void
    {
        $this->featureBuilder->updateStatus($feature, true);
    }

    public function disable(Feature $feature): void
    {
        $this->featureBuilder->updateStatus($feature, false);
    }

    public function enableForProducer(Feature $feature)
    {
        $this->featureProducerBuilder->updateStatusByFeature($feature, true);
    }

    public function disableForProducer(Feature $feature)
    {
        $this->featureProducerBuilder->updateStatusByFeature($feature, false);
    }

    public function defaultForProducer(Feature $feature)
    {
        $this->featureProducerBuilder->updateStatusByFeature($feature, null);
    }
}